@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -19,7 +19,12 @@
 **/
 
 
-$this->includeJSfile('app/views/administration.proxy.edit.js.php');
+/**
+ * @var CView $this
+ */
+
+$this->includeJsFile('administration.proxy.edit.js.php');
+$this->addJsFile('class.tab-indicators.js');
 
 $widget = (new CWidget())->setTitle(_('Proxies'));
 
@@ -30,15 +35,21 @@ if ($data['form_refresh'] == 0) {
 }
 
 $proxyForm = (new CForm())
-	->setId('proxyForm')
+	->setId('proxy-form')
+	->setName('proxy-form')
 	->addVar('proxyid', $data['proxyid'])
 	->addVar('tls_accept', $data['tls_accept'])
+	->addVar('psk_edit_mode', $data['psk_edit_mode'])
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE);
 
 if ($data['status'] == HOST_STATUS_PROXY_PASSIVE && array_key_exists('interfaceid', $data)) {
 	$proxyForm->addVar('interfaceid', $data['interfaceid']);
 }
 
+if (array_key_exists('clone_proxyid', $data)) {
+	$proxyForm->addVar('clone_proxyid', $data['clone_proxyid']);
+}
+
 $interfaceTable = (new CTable())
 	->setHeader([_('IP address'), _('DNS name'), _('Connect to'), _('Port')])
 	->addRow([
@@ -46,8 +57,8 @@ $interfaceTable = (new CTable())
 		(new CTextBox('dns', $data['dns'], false, DB::getFieldLength('interface', 'dns')))
 			->setWidth(ZBX_TEXTAREA_INTERFACE_DNS_WIDTH),
 		(new CRadioButtonList('useip', (int) $data['useip']))
-			->addValue(_('IP'), INTERFACE_USE_IP)
-			->addValue(_('DNS'), INTERFACE_USE_DNS)
+			->addValue('IP', INTERFACE_USE_IP)
+			->addValue('DNS', INTERFACE_USE_DNS)
 			->setModern(true),
 		(new CTextBox('port', $data['port'], false, 64))
 			->setWidth(ZBX_TEXTAREA_INTERFACE_PORT_WIDTH)
@@ -97,17 +108,35 @@ $encryption_form_list = (new CFormList('encryption'))
 			->addItem((new CCheckBox('tls_in_none'))->setLabel(_('No encryption')))
 			->addItem((new CCheckBox('tls_in_psk'))->setLabel(_('PSK')))
 			->addItem((new CCheckBox('tls_in_cert'))->setLabel(_('Certificate')))
-	)
-	->addRow((new CLabel(_('PSK identity'), 'tls_psk_identity'))->setAsteriskMark(),
-		(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], false, 128))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired()
-	)
-	->addRow((new CLabel(_('PSK'), 'tls_psk'))->setAsteriskMark(),
-		(new CTextBox('tls_psk', $data['tls_psk'], false, 512))
-			->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
-			->setAriaRequired()
-	)
+	);
+
+if ($data['psk_edit_mode']) {
+	$encryption_form_list
+		->addRow((new CLabel(_('PSK identity'), 'tls_psk_identity'))->setAsteriskMark(),
+			(new CTextBox('tls_psk_identity', $data['tls_psk_identity'], false, 128))
+				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+				->setAriaRequired()
+		)
+		->addRow((new CLabel(_('PSK'), 'tls_psk'))->setAsteriskMark(),
+			(new CTextBox('tls_psk', $data['tls_psk'], false, 512))
+				->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
+				->setAriaRequired()
+				->disableAutocomplete()
+		);
+}
+else {
+	$encryption_form_list
+		->addRow(
+			(new CLabel(_('PSK')))->setAsteriskMark(),
+			(new CSimpleButton(_('Change PSK')))
+				->setId('change_psk')
+				->addClass(ZBX_STYLE_BTN_GREY),
+			null,
+			'tls_psk'
+		);
+}
+
+$encryption_form_list
 	->addRow(_('Issuer'),
 		(new CTextBox('tls_issuer', $data['tls_issuer'], false, 1024))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 	)
@@ -116,10 +145,13 @@ $encryption_form_list = (new CFormList('encryption'))
 	);
 
 $tabs->addTab('proxyTab', _('Proxy'), $proxy_form_list);
-$tabs->addTab('encryptionTab', _('Encryption'), $encryption_form_list);
+$tabs->addTab('encryptionTab', _('Encryption'), $encryption_form_list, TAB_INDICATOR_ENCRYPTION);
 
 // append buttons to form
-$cancelButton = new CRedirectButton(_('Cancel'), 'zabbix.php?action=proxy.list');
+$cancelButton = (new CRedirectButton(_('Cancel'), (new CUrl('zabbix.php'))
+	->setArgument('action', 'proxy.list')
+	->setArgument('page', CPagerHelper::loadPage('proxy.list', null))
+))->setId('cancel');
 
 if ($data['proxyid'] == 0) {
 	$tabs->setFooter(makeFormFooter(
