@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2019 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,16 @@
  */
 class CHostGroup extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massadd' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massupdate' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'massremove' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'hstgrp';
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['groupid', 'name'];
@@ -68,7 +78,6 @@ class CHostGroup extends CApiService {
 			'with_monitored_httptests'				=> null,
 			'with_graphs'							=> null,
 			'with_graph_prototypes'					=> null,
-			'with_applications'						=> null,
 			'editable'								=> false,
 			'nopermissions'							=> null,
 			// filter
@@ -283,12 +292,6 @@ class CHostGroup extends CApiService {
 			);
 		}
 
-		// with_applications
-		if ($options['with_applications'] !== null) {
-			$sub_sql_parts['from']['a'] = 'applications a';
-			$sub_sql_parts['where']['hg-a'] = 'hg.hostid=a.hostid';
-		}
-
 		if ($sub_sql_parts) {
 			$sub_sql_parts['from']['hg'] = 'hosts_groups hg';
 			$sub_sql_parts['where']['g-hg'] = 'g.groupid=hg.groupid';
@@ -355,7 +358,7 @@ class CHostGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($group = DBfetch($res)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -397,23 +400,40 @@ class CHostGroup extends CApiService {
 		$parent_names = [];
 
 		foreach ($groups as $group) {
-			if (($pos = strrpos($group['name'], '/')) === false) {
-				continue;
-			}
+			$name = $group['name'];
 
-			$parent_names[substr($group['name'], 0, $pos)][] = $group['groupid'];
+			while (($pos = strrpos($name, '/')) !== false) {
+				$name = substr($name, 0, $pos);
+				$parent_names[$name][] = $group['groupid'];
+			}
 		}
 
 		if ($parent_names) {
-			$db_parent_groups = DB::select('hstgrp', [
+			$options = [
 				'output' => ['groupid', 'name'],
 				'filter' => ['name' => array_keys($parent_names)]
-			]);
+			];
+			$result = DBselect(DB::makeSql('hstgrp', $options));
+
+			$db_parent_groups = [];
+
+			while ($row = DBfetch($result)) {
+				$db_parent_groups[$row['name']] = $row['groupid'];
+			}
 
 			$parent_groupids = [];
 
-			foreach ($db_parent_groups as $db_parent_group) {
-				$parent_groupids[$db_parent_group['groupid']] = $parent_names[$db_parent_group['name']];
+			foreach ($groups as $group) {
+				$name = $group['name'];
+
+				while (($pos = strrpos($name, '/')) !== false) {
+					$name = substr($name, 0, $pos);
+
+					if (array_key_exists($name, $db_parent_groups)) {
+						$parent_groupids[$db_parent_groups[$name]][] = $group['groupid'];
+						break;
+					}
+				}
 			}
 
 			if ($parent_groupids) {
@@ -450,23 +470,40 @@ class CHostGroup extends CApiService {
 		$parent_names = [];
 
 		foreach ($groups as $group) {
-			if (($pos = strrpos($group['name'], '/')) === false) {
-				continue;
-			}
+			$name = $group['name'];
 
-			$parent_names[substr($group['name'], 0, $pos)][] = $group['groupid'];
+			while (($pos = strrpos($name, '/')) !== false) {
+				$name = substr($name, 0, $pos);
+				$parent_names[$name][] = $group['groupid'];
+			}
 		}
 
 		if ($parent_names) {
-			$db_parent_groups = DB::select('hstgrp', [
+			$options = [
 				'output' => ['groupid', 'name'],
 				'filter' => ['name' => array_keys($parent_names)]
-			]);
+			];
+			$result = DBselect(DB::makeSql('hstgrp', $options));
+
+			$db_parent_groups = [];
+
+			while ($row = DBfetch($result)) {
+				$db_parent_groups[$row['name']] = $row['groupid'];
+			}
 
 			$parent_groupids = [];
 
-			foreach ($db_parent_groups as $db_parent_group) {
-				$parent_groupids[$db_parent_group['groupid']] = $parent_names[$db_parent_group['name']];
+			foreach ($groups as $group) {
+				$name = $group['name'];
+
+				while (($pos = strrpos($name, '/')) !== false) {
+					$name = substr($name, 0, $pos);
+
+					if (array_key_exists($name, $db_parent_groups)) {
+						$parent_groupids[$db_parent_groups[$name]][] = $group['groupid'];
+						break;
+					}
+				}
 			}
 
 			if ($parent_groupids) {
@@ -555,19 +592,6 @@ class CHostGroup extends CApiService {
 	public function delete(array $groupids, $nopermissions = false) {
 		$this->validateDelete($groupids, $db_groups, $nopermissions);
 
-		// delete screens items
-		$resources = [
-			SCREEN_RESOURCE_HOSTGROUP_TRIGGERS,
-			SCREEN_RESOURCE_HOST_INFO,
-			SCREEN_RESOURCE_TRIGGER_INFO,
-			SCREEN_RESOURCE_TRIGGER_OVERVIEW,
-			SCREEN_RESOURCE_DATA_OVERVIEW
-		];
-		DB::delete('screens_items', [
-			'resourceid' => $groupids,
-			'resourcetype' => $resources
-		]);
-
 		// delete sysmap element
 		if (!empty($groupids)) {
 			DB::delete('sysmaps_elements', ['elementtype' => SYSMAP_ELEMENT_TYPE_HOST_GROUP, 'elementid' => $groupids]);
@@ -695,7 +719,8 @@ class CHostGroup extends CApiService {
 			self::exception(ZBX_API_ERROR_PERMISSIONS, _('Only Super Admins can create host groups.'));
 		}
 
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['name']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['name']], 'fields' => [
+			'uuid' =>	['type' => API_UUID],
 			'name' =>	['type' => API_HG_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hstgrp', 'name')]
 		]];
 		if (!CApiInputValidator::validate($api_input_rules, $groups, '/', $error)) {
@@ -703,6 +728,35 @@ class CHostGroup extends CApiService {
 		}
 
 		$this->checkDuplicates(zbx_objectValues($groups, 'name'));
+		$this->checkAndAddUuid($groups);
+	}
+
+	/**
+	 * Check that new UUIDs are not already used and generate UUIDs where missing.
+	 *
+	 * @param array $groups_to_create
+	 *
+	 * @throws APIException
+	 */
+	protected function checkAndAddUuid(array &$groups_to_create): void {
+		foreach ($groups_to_create as &$group) {
+			if (!array_key_exists('uuid', $group)) {
+				$group['uuid'] = generateUuidV4();
+			}
+		}
+		unset($group);
+
+		$db_uuid = DB::select('hstgrp', [
+			'output' => ['uuid'],
+			'filter' => ['uuid' => array_column($groups_to_create, 'uuid')],
+			'limit' => 1
+		]);
+
+		if ($db_uuid) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Entry with UUID "%1$s" already exists.', $db_uuid[0]['uuid'])
+			);
+		}
 	}
 
 	/**
@@ -773,23 +827,18 @@ class CHostGroup extends CApiService {
 
 		$this->verifyHostsAndTemplatesAreUnlinkable($hosts_to_unlink, $templates_to_unlink, $groupids);
 
-		$db_scripts = API::Script()->get([
+		$db_scripts = DB::select('scripts', [
+			'output' => ['groupid'],
 			'filter' => ['groupid' => $groupids],
-			'output' => ['scriptid', 'groupid'],
-			'nopermissions' => true
+			'limit' => 1
 		]);
 
-		if (!empty($db_scripts)) {
-			foreach ($db_scripts as $script) {
-				if ($script['groupid'] == 0) {
-					continue;
-				}
-				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
-						$db_groups[$script['groupid']]['name']
-					)
-				);
-			}
+		if ($db_scripts) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('Host group "%1$s" cannot be deleted, because it is used in a global script.',
+					$db_groups[$db_scripts[0]['groupid']]['name']
+				)
+			);
 		}
 
 		$corr_condition_group = DBFetch(DBselect(
