@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2021 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -24,6 +24,13 @@
  */
 class CUserGroup extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_SUPER_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_SUPER_ADMIN]
+	];
+
 	protected $tableName = 'usrgrp';
 	protected $tableAlias = 'g';
 	protected $sortColumns = ['usrgrpid', 'name'];
@@ -139,7 +146,7 @@ class CUserGroup extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$res = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$res = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
 		while ($usrgrp = DBfetch($res)) {
 			if ($options['countOutput']) {
 				$result = $usrgrp['rowscount'];
@@ -530,6 +537,7 @@ class CUserGroup extends CApiService {
 
 		foreach ($usrgrps as $usrgrp) {
 			if (self::userGroupDisabled($usrgrp, $method, $db_usrgrps)
+					&& array_key_exists('userids', $usrgrp)
 					&& uint_in_array(self::$userData['userid'], $usrgrp['userids'])) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_('User cannot add himself to a disabled group or a group with disabled GUI access.')
@@ -597,17 +605,17 @@ class CUserGroup extends CApiService {
 		}
 
 		$db_users = DBselect(
-			'SELECT u.userid,u.alias,count(ug.usrgrpid) as usrgrp_num'.
+			'SELECT u.userid,u.username,count(ug.usrgrpid) as usrgrp_num'.
 			' FROM users u,users_groups ug'.
 			' WHERE u.userid=ug.userid'.
 				' AND '.dbConditionInt('u.userid', array_keys($del_userids)).
-			' GROUP BY u.userid,u.alias'
+			' GROUP BY u.userid,u.username'
 		);
 
 		while ($db_user = DBfetch($db_users)) {
 			if ($db_user['usrgrp_num'] == $del_userids[$db_user['userid']]) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
-					_s('User "%1$s" cannot be without user group.', $db_user['alias'])
+					_s('User "%1$s" cannot be without user group.', $db_user['username'])
 				);
 			}
 		}
@@ -653,7 +661,7 @@ class CUserGroup extends CApiService {
 				if ($db_right['permission'] != $rights[$db_right['groupid']][$db_right['id']]) {
 					$upd_rights[] = [
 						'values' => ['permission' => $rights[$db_right['groupid']][$db_right['id']]],
-						'where' => ['rightid' => $db_right['rightid']],
+						'where' => ['rightid' => $db_right['rightid']]
 					];
 				}
 				unset($rights[$db_right['groupid']][$db_right['id']]);
@@ -913,12 +921,24 @@ class CUserGroup extends CApiService {
 		}
 
 		// Check if user group are used in config.
-		$config = select_config();
+		if (array_key_exists(CSettingsHelper::get(CSettingsHelper::ALERT_USRGRPID), $db_usrgrps)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS,
+				_s('User group "%1$s" is used in configuration for database down messages.', $db_usrgrps[CSettingsHelper::get(CSettingsHelper::ALERT_USRGRPID)]['name'])
+			);
+		}
 
-		if (array_key_exists($config['alert_usrgrpid'], $db_usrgrps)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _s(
-				'User group "%1$s" is used in configuration for database down messages.',
-				$db_usrgrps[$config['alert_usrgrpid']]['name']
+		// Check if user groups are used in scheduled reports.
+		$db_reports = DBselect(
+			'SELECT r.name,rug.usrgrpid'.
+			' FROM report r,report_usrgrp rug'.
+			' WHERE r.reportid=rug.reportid'.
+				' AND '.dbConditionInt('rug.usrgrpid', $usrgrpids),
+			1
+		);
+
+		if ($db_report = DBfetch($db_reports)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('User group "%1$s" is report "%2$s" recipient.',
+				$db_usrgrps[$db_report['usrgrpid']]['name'], $db_report['name']
 			));
 		}
 
@@ -932,10 +952,14 @@ class CUserGroup extends CApiService {
 		if ($options['selectUsers'] !== null && $options['selectUsers'] != API_OUTPUT_COUNT) {
 			$relationMap = $this->createRelationMap($result, 'usrgrpid', 'userid', 'users_groups');
 
+			$get_access = ($this->outputIsRequested('gui_access', $options['selectUsers'])
+				|| $this->outputIsRequested('debug_mode', $options['selectUsers'])
+				|| $this->outputIsRequested('users_status', $options['selectUsers'])) ? true : null;
+
 			$dbUsers = API::User()->get([
 				'output' => $options['selectUsers'],
 				'userids' => $relationMap->getRelatedIds(),
-				'getAccess' => ($options['selectUsers'] == API_OUTPUT_EXTEND) ? true : null,
+				'getAccess' => $get_access,
 				'preservekeys' => true
 			]);
 
